/* Use the UAL syntax */
        .syntax unified

/* Symbols declared in the linker script */
        .global _sp_init
        .global _bss_section_start
        .global _bss_section_end
        .global _data_section_start
        .global _data_section_end
        .global _data_section_lma

/* Main function of the program to which this script is linked */
        .global main

/* ARM vector table (aka isr_vector) */
        .section .vector_table
        .align 2
        .word _sp_init /* Initialisation value of the stack pointer register */
/* Exceptions handlers */
        .word reset_handler /* Reset Exception */
        .word nmi_handler /* NMI Exception */
        .word hardfault_handler /* HardFault Exception */
        .word memmanager_handler /* MemManage Exception */
        .word busfault_handler /* BusFault Exception */
        .word usagefault_handler /* UsageFault Exception */
        .word 0 /* Reserved */
        .word 0 /* Reserved */
        .word 0 /* Reserved */
        .word 0 /* Reserved */
        .word svcall_handler /* SVCall Exception */
        .word debugmonitor_handler /* DebugMonitor Exception */
        .word 0 /* Reserved */
        .word pendsv_handler /* PendSV Exception */
        .word systick_handler /* SysTick Exception */

        .text
        .align

        .global default_exception_handler
        .thumb_func
default_exception_handler:
        b .

/* The CPU Reset Exception Handler, initializing register and data section, then call the main function */
        .global reset_handler
        .thumb_func
reset_handler:
/* Initialize the CPU general purpose registers */
        mov r0, 0
        mov r1, 0
        mov r2, 0
        mov r3, 0
        mov r4, 0
        mov r5, 0
        mov r6, 0
        mov r7, 0
        mov r8, 0
        mov r9, 0
        mov r10, 0
        mov r11, 0
        mov r12, 0

/* .bss section initialization (see linker script) */
bss_init:
        mov r0, 0
        ldr r1, =_bss_section_start
        ldr r2, =_bss_section_end

bss_init_loop:
/* Fill the section word-by-word with zeros */
        cmp r1, r2
        beq data_init
        str r0, [r1], 4
        b bss_init_loop

/* Data section initialization */
data_init:
        ldr r1, =_data_section_start
        ldr r2, =_data_section_end
        ldr r3, =_data_section_lma

/* Load the initial values of the variables from the section LMA */
data_init_loop:
        cmp r1, r2
        beq bootstrap_end
        ldr r0, [r3], 4
        str r0, [r1], 4
        b data_init_loop

/* Bootstrap completed, branch to Rust code now ! */
bootstrap_end:
        bl main
